$fn 			= 90;	// set the quality here
ringHeight 		= 7;	// Height of the ring
ringDiameter 	= 22;	// Inner diameter of the ring (size of your finger). Adds 1mm to create thickness.


// *************** DO NOT EDIT ANYTHING BELOW THIS LINE ********** //
// ***** Advanced options ***** //
_numberOfStuds 		= 6;		// Number of studs around the ring
_studRadiusFactor		= 20;		// The studs will be scaled down to this percentage of the ring height
_torusRadius 			= 0.75;		// The radius of the torus.
_ringThickness 		= 1;		// The ring thickness. If you increase this, 
									// you should also increase _torusRadius to about 75% of this value
// ***** End of Advanced Options ***** //

generateRing(); // Because I do not want a union() outside of a module...

module generateRing() {
	/*
		This function will generate all the parts of the ring and union() them. The parts are:
		1. The basic ring
		2. The studs
		3. The rounded top and bottom pieces
	*/
	union() {
		BasicRing(ringDiameter, ringHeight); 		// The basic ring shape
		for(z = [0:_numberOfStuds - 1]) {		// Each of the studs, taking into consideration _numberOfStuds
			rotate(z * 360 / _numberOfStuds) 
				Stud(ringDiameter, ringHeight);
		}
		Torus(_torusRadius, ringDiameter);		// The bottom torus
		translate([0, 0, ringHeight]) 
			Torus(_torusRadius, ringDiameter);	// The top torus
	}
}

module BasicRing(innerDiameter, height) {
	// Draw a basic ring shape given the parameters
	iDia = innerDiameter / 2;
	difference() {
		cylinder(height, iDia + _ringThickness, iDia + _ringThickness);
		translate([0, 0, -0.1]) 
			cylinder(height+0.2, iDia, iDia);
	}
}

module Stud(ringInnerDiameter, ringHeight) {
	// Draw a partial sphere
	iDia = (ringInnerDiameter / 2) + (_ringThickness - 0.1);
	difference() {
		translate([iDia, 0, ringHeight/2]) 
			sphere(ringHeight * (_studRadiusFactor / 100));
		translate([0, 0, -0.1]) 
			cylinder(ringHeight + 0.2, iDia, iDia);
	}
}

module Torus(radius, ringInnerDiameter) {
	rotate_extrude(convexity = 10)
		translate([(ringInnerDiameter / 2) + _torusRadius, 0, 0])
			circle(r = radius);
}